@@ -34,6 +34,7 @@
 #include <osmocom/vty/logging.h>
 #include <osmocom/vty/telnet_interface.h>
 #include <osmocom/vty/stats.h>
+#include <osmocom/vty/misc.h>
 
 #include <pcap.h>
 #include <signal.h>
@@ -103,25 +104,58 @@ static void print_help()
 	printf("  Some useful help...\n");
 	printf("  -h --help this text\n");
 	printf("  -D --daemonize Fork the process into a background daemon\n");
+	printf("  -V --version Print the version number\n");
 	printf("  -d option --debug=DRLL:DCC:DMM:DRR:DRSL:DNM enable debugging\n");
 	printf("  -s --disable-color\n");
 	printf("  -T --timestamp. Print a timestamp in the debug output.\n");
 	printf("  -e --log-level number. Set a global loglevel.\n");
 	printf("  -c --config-file filename The config file to use.\n");
+
+	printf("\nVTY reference generation:\n");
+	printf("     --vty-ref-mode MODE     VTY reference generation mode (e.g. 'expert').\n");
+	printf("     --vty-ref-xml           Generate the VTY reference XML output and exit.\n");
+}
+
+static void handle_long_options(const char *prog_name, const int long_option)
+{
+	static int vty_ref_mode = VTY_REF_GEN_MODE_DEFAULT;
+	switch (long_option) {
+	case 1:
+		vty_ref_mode = get_string_value(vty_ref_gen_mode_names, optarg);
+		if (vty_ref_mode < 0) {
+			fprintf(stderr, "%s: Unknown VTY reference generation "
+				"mode '%s'\n", prog_name, optarg);
+			exit(2);
+		}
+		break;
+	case 2:
+		fprintf(stderr, "Generating the VTY reference in mode '%s' (%s)\n",
+			get_value_string(vty_ref_gen_mode_names, vty_ref_mode),
+			get_value_string(vty_ref_gen_mode_desc, vty_ref_mode));
+		vty_dump_xml_ref_mode(stdout, (enum vty_ref_gen_mode) vty_ref_mode);
+		exit(0);
+	default:
+		fprintf(stderr, "%s: error parsing cmdline options\n", prog_name);
+		exit(2);
+	}
 }
 
 static void handle_options(int argc, char **argv)
 {
 	while (1) {
 		int option_index = 0, c;
+		static int long_option = 0;
 		static struct option long_options[] = {
 			{"help", 0, 0, 'h'},
 			{"daemonize", 0, 0, 'D'},
 			{"debug", 1, 0, 'd'},
+			{"version", 0, 0, 'V'},
 			{"disable-color", 0, 0, 's'},
 			{"timestamp", 0, 0, 'T'},
 			{"log-level", 1, 0, 'e'},
 			{"config-file", 1, 0, 'c'},
+			{"vty-ref-mode", 1, &long_option, 1},
+			{"vty-ref-xml", 0, &long_option, 2},
 			{0, 0, 0, 0}
 		};
 
@@ -135,12 +169,18 @@ static void handle_options(int argc, char **argv)
 			print_usage();
 			print_help();
 			exit(0);
+		case 0:
+			handle_long_options(argv[0], long_option);
+			break;
 		case 'D':
 			daemonize = 1;
 			break;
 		case 'd':
 			log_parse_category_mask(osmo_stderr_target, optarg);
 			break;
+		case 'V':
+			print_version(1);
+			exit(0);
 		case 's':
 			log_set_use_color(osmo_stderr_target, 0);
 			break;
@@ -207,9 +247,12 @@ int main(int argc, char **argv)
 	osmo_init_logging2(tall_srv_ctx, &log_info);
 
 	vty_info.copyright = osmopcap_copyright;
+	vty_info.tall_ctx = tall_srv_ctx;
 	vty_init(&vty_info);
 	logging_vty_add_cmds();
 	osmo_stats_vty_add_cmds();
+	osmo_talloc_vty_add_cmds();
+	vty_server_init();
 
 	/* parse options */
 	handle_options(argc, argv);
@@ -228,12 +271,6 @@ int main(int argc, char **argv)
 
 	osmo_tls_init();
 
-	rc = telnet_init(tall_srv_ctx, NULL, OSMO_VTY_PORT_PCAP_SERVER);
-	if (rc < 0) {
-		LOGP(DCLIENT, LOGL_ERROR, "Failed to bind telnet interface\n");
-		exit(1);
-	}
-
 	pcap_server = talloc_zero(tall_srv_ctx, struct osmo_pcap_server);
 	if (!pcap_server) {
 		LOGP(DSERVER, LOGL_ERROR, "Failed to allocate osmo_pcap_server.\n");
@@ -249,7 +286,6 @@ int main(int argc, char **argv)
 	pcap_server->base_path = talloc_strdup(pcap_server, "./");
 	pcap_server->max_size = 1073741824;
 	pcap_server->max_snaplen = DEFAULT_SNAPLEN;
-	vty_server_init(pcap_server);
 
 	if (vty_read_config_file(config_file, NULL) < 0) {
 		LOGP(DSERVER, LOGL_ERROR,
@@ -257,6 +293,12 @@ int main(int argc, char **argv)
 		exit(1);
 	}
 
+	rc = telnet_init_dynif(tall_srv_ctx, NULL, vty_get_bind_addr(), OSMO_VTY_PORT_PCAP_SERVER);
+	if (rc < 0) {
+		LOGP(DCLIENT, LOGL_ERROR, "Failed to bind telnet interface\n");
+		exit(1);
+	}
+
 	osmo_tls_server_init(pcap_server);
 
 	/* attempt to connect to the remote */
